import type { NitroFetchOptions } from 'nitropack';
import type { AllPaths, ApiResponse, CaseVariants, GetPaths, GetPlainPaths, HttpMethod, RequestOptions, SchemaPath } from '../openapi';
export interface BaseApiFetchOptions {
    /**
     * Skip the Nuxt server proxy and fetch directly from the API.
     * Requires `client` set to `true` in the module options.
     * @remarks
     * If Nuxt SSR is disabled, client-side requests are enabled by default.
     * @default false
     */
    client?: boolean;
    /**
     * Cache the response for the same request.
     * You can customize the cache key with the `key` option.
     * @default false
     */
    cache?: boolean;
    /**
     * By default, a cache key will be generated from the request options.
     * With this option, you can provide a custom cache key.
     * @default undefined
     */
    key?: string;
}
export type ApiFetchOptions = Omit<NitroFetchOptions<string>, 'body' | 'cache'> & {
    pathParams?: Record<string, string>;
    body?: string | Record<string, any> | FormData | null;
};
export type $Api = <T = any>(path: string, opts?: ApiFetchOptions & BaseApiFetchOptions) => Promise<T>;
export interface $OpenAPI<Paths extends Record<string, SchemaPath>> {
    <P extends GetPlainPaths<Paths>>(path: P, opts?: BaseApiFetchOptions & Omit<RequestOptions<Paths[`/${P}`]>, 'method'>): Promise<ApiResponse<Paths[`/${P}`]['get']>>;
    <P extends GetPaths<Paths>>(path: P, opts: BaseApiFetchOptions & Omit<RequestOptions<Paths[`/${P}`]>, 'method'>): Promise<ApiResponse<Paths[`/${P}`]['get']>>;
    <P extends AllPaths<Paths>, M extends CaseVariants<keyof Paths[`/${P}`] & HttpMethod>>(path: P, opts?: BaseApiFetchOptions & RequestOptions<Paths[`/${P}`], M> & {
        method: M;
    }): Promise<ApiResponse<Paths[`/${P}`][Lowercase<M>]>>;
}
export declare function _$api<T = any>(endpointId: string, path: string, opts?: ApiFetchOptions & BaseApiFetchOptions): Promise<any>;
