import type { FetchError } from 'ofetch';
import type { NitroFetchOptions } from 'nitropack';
import type { WatchSource } from 'vue';
import type { AsyncData, AsyncDataOptions } from 'nuxt/app';
import type { MaybeRef, MaybeRefOrGetter } from '../types';
import type { AllPaths, ApiError, ApiResponse, CaseVariants, GetPaths, GetPlainPaths, HttpMethod, RequestOptions, SchemaPath } from '../openapi';
type ComputedOptions<T extends Record<string, any>> = {
    [K in keyof T]: T[K] extends (...args: any[]) => any ? T[K] : T[K] extends Record<string, any> ? ComputedOptions<T[K]> | MaybeRef<T[K]> : MaybeRef<T[K]>;
};
export type BaseUseApiDataOptions<ResT, DataT = ResT> = Omit<AsyncDataOptions<ResT, DataT>, 'watch'> & {
    /**
     * Skip the Nuxt server proxy and fetch directly from the API.
     * Requires `client` set to `true` in the module options.
     * @remarks
     * If Nuxt SSR is disabled, client-side requests are enabled by default.
     * @default false
     */
    client?: boolean;
    /**
     * Cache the response for the same request.
     * You can customize the cache key with the `key` option.
     * @default true
     */
    cache?: boolean;
    /**
     * By default, a cache key will be generated from the request options.
     * With this option, you can provide a custom cache key.
     * @default undefined
     */
    key?: MaybeRefOrGetter<string>;
    /**
     * Watch an array of reactive sources and auto-refresh the fetch result when they change.
     * Fetch options and URL are watched by default. You can completely ignore reactive sources by using `watch: false`.
     * @default undefined
     */
    watch?: (WatchSource<unknown> | object)[] | false;
};
export type UseApiDataOptions<T> = Pick<ComputedOptions<NitroFetchOptions<string>>, 'onRequest' | 'onRequestError' | 'onResponse' | 'onResponseError' | 'query' | 'headers' | 'method' | 'retry' | 'retryDelay' | 'timeout'> & {
    pathParams?: MaybeRefOrGetter<Record<string, string>>;
    body?: MaybeRef<string | Record<string, any> | FormData | null>;
} & BaseUseApiDataOptions<T>;
export type UseOpenAPIDataOptions<P extends SchemaPath, M extends CaseVariants<keyof P & HttpMethod> = CaseVariants<keyof P & 'get'>, ResT = ApiResponse<P[Lowercase<M>]>, DataT = ResT> = BaseUseApiDataOptions<ResT, DataT> & ComputedOptions<RequestOptions<P, M>>;
export type UseApiData = <T = any>(path: MaybeRefOrGetter<string>, opts?: UseApiDataOptions<T>) => AsyncData<T | undefined, FetchError>;
export interface UseOpenAPIData<Paths extends Record<string, SchemaPath>> {
    <P extends GetPlainPaths<Paths>, ResT = ApiResponse<Paths[`/${P}`]['get']>, DataT = ResT>(path: MaybeRefOrGetter<P>, opts?: Omit<UseOpenAPIDataOptions<Paths[`/${P}`], CaseVariants<keyof Paths[`/${P}`] & HttpMethod>, ResT, DataT>, 'method'>): AsyncData<DataT, FetchError<ApiError<Paths[`/${P}`]['get']>>>;
    <P extends GetPaths<Paths>, ResT = ApiResponse<Paths[`/${P}`]['get']>, DataT = ResT>(path: MaybeRefOrGetter<P>, opts: Omit<UseOpenAPIDataOptions<Paths[`/${P}`], CaseVariants<keyof Paths[`/${P}`] & HttpMethod>, ResT, DataT>, 'method'>): AsyncData<DataT, FetchError<ApiError<Paths[`/${P}`]['get']>>>;
    <P extends AllPaths<Paths>, M extends CaseVariants<keyof Paths[`/${P}`] & HttpMethod>, ResT = ApiResponse<Paths[`/${P}`][Lowercase<M>]>, DataT = ResT>(path: MaybeRefOrGetter<P>, opts: UseOpenAPIDataOptions<Paths[`/${P}`], M, ResT, DataT> & {
        method: M;
    }): AsyncData<DataT, FetchError<ApiError<Paths[`/${P}`][Lowercase<M>]>>>;
}
export declare function _useApiData<T = any>(endpointId: string, path: MaybeRefOrGetter<string>, opts?: UseApiDataOptions<T>): AsyncData<T | undefined, FetchError<any>>;
export {};
