import { unref } from "vue";
import { formDataToObject, isFormData, isSerializedFormData, objectToFormData } from "./formData.mjs";
export function toValue(r) {
  return typeof r === "function" ? r() : unref(r);
}
export function headersToObject(headers = {}) {
  if (headers instanceof Headers || Array.isArray(headers))
    return Object.fromEntries(headers);
  return headers;
}
export async function serializeMaybeEncodedBody(value) {
  if (isFormData(value))
    return await formDataToObject(value);
  return value;
}
export async function deserializeMaybeEncodedBody(value) {
  if (isSerializedFormData(value))
    return await objectToFormData(value);
  return value;
}
export function resolvePathParams(path, params) {
  if (params) {
    for (const [key, value] of Object.entries(params))
      path = path.replace(`{${key}}`, encodeURIComponent(String(toValue(value))));
  }
  return path;
}
